@@ -1,10 +1,11 @@
-setMethodS3("allocateFromCdf", "AromaCellCpgFile", function(static, cdf, path=getPath(cdf), tags="*", ...) {
+setMethodS3("allocateFromCdf", "AromaCellCpgFile", function(static, cdf, path=NULL, tags="*", ...) {
   # Argument 'cdf':
   cdf <- Arguments$getInstanceOf(cdf, "AffymetrixCdfFile");
 
   # Argument 'tags':
   tags <- Arguments$getTags(tags, collapse=NULL);
 
+
   chipType <- getChipType(cdf);
   parts <- strsplit(chipType, split=",", fixed=TRUE);
   parts <- unlist(parts, use.names=FALSE);
@@ -15,6 +16,12 @@ setMethodS3("allocateFromCdf", "AromaCellCpgFile", function(static, cdf, path=ge
   # Drop unwanted tags. AD HOC. /HB 2010-02-22
   chipType <- gsub(",monocell", "", chipType, fixed=TRUE);
 
+  # Output path
+  if (is.null(path)) {
+    path <- file.path("annotationData", "chipTypes", chipType);
+  }
+  path <- Arguments$getWritablePath(path);
+
   platform <- getPlatform(cdf);
   nbrOfCells <- nbrOfCells(cdf);
   fullname <- paste(c(chipType, tags), collapse=",");
@@ -28,6 +35,11 @@ setMethodS3("allocateFromCdf", "AromaCellCpgFile", function(static, cdf, path=ge
 
 ############################################################################
 # HISTORY:
+# 2011-02-28
+# o STANDARDIZATION: Now the default output path for all allocateFromCdf()
+#   is annotationData/chipTypes/<chipType>/.  Before it was the same
+#   directory as the CDF, which may for instance have been in a deeper
+#   subdirectory, or recently also in a sibling root path.
 # 2010-03-14 [HB]
 # o BUG FIX: allocateFromCdf() of AromaCellCpgFile would drop all but the
 #   first tag.
