@@ -120,8 +120,7 @@ setMethodS3("fromDataFile", "ResidualFile", function(static, df=NULL, filename=s
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   # Argument 'df':
   if (!is.null(df)) {
-    if (!inherits(df, "AffymetrixCelFile"))
-      throw("Argument 'df' is not an AffymetrixCelFile: ", class(df)[1]);
+    df <- Arguments$getInstanceOf(df, "AffymetrixCelFile");
   }
 
   # Argument 'cdf':
@@ -129,8 +128,7 @@ setMethodS3("fromDataFile", "ResidualFile", function(static, df=NULL, filename=s
     if (is.null(df))
       throw("Either argument 'df' or 'cdf' must specified.");
   } else {
-    if (!inherits(cdf, "AffymetrixCdfFile"))
-      throw("Argument 'cdf' is not an AffymetrixCdfFile: ", class(cdf)[1]);
+    cdf <- Arguments$getInstanceOf(cdf, "AffymetrixCdfFile");
   }
 
   # Argument 'filename' & 'path':
